@@ -47,9 +47,9 @@ class OC_Calendar_Calendar{
 
 		$calendars = array();
 		while( $row = $result->fetchRow()) {
-			$row['permissions'] = OCP\Share::PERMISSION_CREATE
-				| OCP\Share::PERMISSION_READ | OCP\Share::PERMISSION_UPDATE
-				| OCP\Share::PERMISSION_DELETE | OCP\Share::PERMISSION_SHARE;
+			$row['permissions'] = OCP\PERMISSION_CREATE
+				| OCP\PERMISSION_READ | OCP\PERMISSION_UPDATE
+				| OCP\PERMISSION_DELETE | OCP\PERMISSION_SHARE;
 			$calendars[] = $row;
 		}
 		$calendars = array_merge($calendars, OCP\Share::getItemsSharedWith('calendar', OC_Share_Backend_Calendar::FORMAT_CALENDAR));
@@ -70,26 +70,21 @@ class OC_Calendar_Calendar{
 	/**
 	 * @brief Gets the data of one calendar
 	 * @param integer $id
-	 * @return associative array or false if entry wasn't found
+	 * @return associative array
 	 */
 	public static function find($id) {
 		$stmt = OCP\DB::prepare( 'SELECT * FROM `*PREFIX*calendar_calendars` WHERE `id` = ?' );
 		$result = $stmt->execute(array($id));
 
-		if($result->numRows() == 0) {
-			return false;
-		}
 		$row = $result->fetchRow();
 		if($row['userid'] != OCP\USER::getUser() && !OC_Group::inGroup(OCP\User::getUser(), 'admin')) {
 			$sharedCalendar = OCP\Share::getItemSharedWithBySource('calendar', $id);
-			if (!$sharedCalendar || !($sharedCalendar['permissions'] & OCP\Share::PERMISSION_READ)) {
+			if (!$sharedCalendar || !($sharedCalendar['permissions'] & OCP\PERMISSION_READ)) {
 				return $row; // I have to return the row so e.g. OC_Calendar_Object::getowner() works.
 			}
 			$row['permissions'] = $sharedCalendar['permissions'];
 		} else {
-			$row['permissions'] = OCP\Share::PERMISSION_CREATE
-				| OCP\Share::PERMISSION_READ | OCP\Share::PERMISSION_UPDATE
-				| OCP\Share::PERMISSION_DELETE | OCP\Share::PERMISSION_SHARE;
+			$row['permissions'] = OCP\PERMISSION_ALL;
 		}
 		return $row;
 	}
@@ -122,6 +117,21 @@ class OC_Calendar_Calendar{
 		return $insertid;
 	}
 
+	/**
+	 * @brief Creates default calendars
+	 * @param string $userid
+	 * @return boolean
+	 */
+	public static function addDefaultCalendars($userid = null) {
+		if(is_null($userid)) {
+			$userid = OCP\USER::getUser();
+		}
+		
+		$id = self::addCalendar($userid,'Default calendar');
+
+		return true;
+	}
+
 	/**
 	 * @brief Creates a new calendar from the data sabredav provides
 	 * @param string $principaluri
@@ -162,7 +172,7 @@ class OC_Calendar_Calendar{
 		$calendar = self::find($id);
 		if ($calendar['userid'] != OCP\User::getUser() && !OC_Group::inGroup(OCP\User::getUser(), 'admin')) {
 			$sharedCalendar = OCP\Share::getItemSharedWithBySource('calendar', $id);
-			if (!$sharedCalendar || !($sharedCalendar['permissions'] & OCP\Share::PERMISSION_UPDATE)) {
+			if (!$sharedCalendar || !($sharedCalendar['permissions'] & OCP\PERMISSION_UPDATE)) {
 				throw new Exception(
 					OC_Calendar_App::$l10n->t(
 						'You do not have the permissions to update this calendar.'
@@ -195,7 +205,7 @@ class OC_Calendar_Calendar{
 		$calendar = self::find($id);
 		if ($calendar['userid'] != OCP\User::getUser()) {
 			$sharedCalendar = OCP\Share::getItemSharedWithBySource('calendar', $id);
-			if (!$sharedCalendar || !($sharedCalendar['permissions'] & OCP\Share::PERMISSION_UPDATE)) {
+			if (!$sharedCalendar || !($sharedCalendar['permissions'] & OCP\PERMISSION_UPDATE)) {
 				throw new Exception(
 					OC_Calendar_App::$l10n->t(
 						'You do not have the permissions to update this calendar.'
@@ -230,7 +240,7 @@ class OC_Calendar_Calendar{
 		$calendar = self::find($id);
 		if ($calendar['userid'] != OCP\User::getUser() && !OC_Group::inGroup(OCP\User::getUser(), 'admin')) {
 			$sharedCalendar = OCP\Share::getItemSharedWithBySource('calendar', $id);
-			if (!$sharedCalendar || !($sharedCalendar['permissions'] & OCP\Share::PERMISSION_DELETE)) {
+			if (!$sharedCalendar || !($sharedCalendar['permissions'] & OCP\PERMISSION_DELETE)) {
 				throw new Exception(
 					OC_Calendar_App::$l10n->t(
 						'You do not have the permissions to delete this calendar.'
@@ -248,7 +258,7 @@ class OC_Calendar_Calendar{
 
 		OCP\Util::emitHook('OC_Calendar', 'deleteCalendar', $id);
 		if(OCP\USER::isLoggedIn() and count(self::allCalendars(OCP\USER::getUser())) == 0) {
-			self::addCalendar(OCP\USER::getUser(),'Default calendar');
+			self::addDefaultCalendars(OCP\USER::getUser());
 		}
 
 		return true;
@@ -264,7 +274,7 @@ class OC_Calendar_Calendar{
 		$calendar = self::find($id1);
 		if ($calendar['userid'] != OCP\User::getUser() && !OC_Group::inGroup(OCP\User::getUser(), 'admin')) {
 			$sharedCalendar = OCP\Share::getItemSharedWithBySource('calendar', $id1);
-			if (!$sharedCalendar || !($sharedCalendar['permissions'] & OCP\Share::PERMISSION_UPDATE)) {
+			if (!$sharedCalendar || !($sharedCalendar['permissions'] & OCP\PERMISSION_UPDATE)) {
 				throw new Exception(
 					OC_Calendar_App::$l10n->t(
 						'You do not have the permissions to add to this calendar.'
