@@ -84,16 +84,12 @@ class OC_Calendar_Object{
 	/**
 	 * @brief Returns an object
 	 * @param integer $id
-	 * @return associative array or false if entry isn't found.
+	 * @return associative array
 	 */
 	public static function find($id) {
 		$stmt = OCP\DB::prepare( 'SELECT * FROM `*PREFIX*calendar_objects` WHERE `id` = ?' );
 		$result = $stmt->execute(array($id));
 
-		if($result->numRows() == 0) {
-			return false;
-		}
-
 		return $result->fetchRow();
 	}
 
@@ -120,7 +116,7 @@ class OC_Calendar_Object{
 		$calendar = OC_Calendar_Calendar::find($id);
 		if ($calendar['userid'] != OCP\User::getUser()) {
 			$sharedCalendar = OCP\Share::getItemSharedWithBySource('calendar', $id);
-			if (!$sharedCalendar || !($sharedCalendar['permissions'] & OCP\Share::PERMISSION_CREATE)) {
+			if (!$sharedCalendar || !($sharedCalendar['permissions'] & OCP\PERMISSION_CREATE)) {
 				throw new Exception(
 					OC_Calendar_App::$l10n->t(
 						'You do not have the permissions to add events to this calendar.'
@@ -129,7 +125,6 @@ class OC_Calendar_Object{
 			}
 		}
 		$object = OC_VObject::parse($data);
-		OC_Calendar_App::loadCategoriesFromVCalendar($object);
 		list($type,$startdate,$enddate,$summary,$repeating,$uid) = self::extractData($object);
 
 		if(is_null($uid)) {
@@ -143,6 +138,8 @@ class OC_Calendar_Object{
 		$stmt->execute(array($id,$type,$startdate,$enddate,$repeating,$summary,$data,$uri,time()));
 		$object_id = OCP\DB::insertid('*PREFIX*calendar_objects');
 
+		OC_Calendar_App::loadCategoriesFromVCalendar($object_id, $object);
+
 		OC_Calendar_Calendar::touchCalendar($id);
 		OCP\Util::emitHook('OC_Calendar', 'addEvent', $object_id);
 		return $object_id;
@@ -159,7 +156,7 @@ class OC_Calendar_Object{
 		$calendar = OC_Calendar_Calendar::find($id);
 		if ($calendar['userid'] != OCP\User::getUser()) {
 			$sharedCalendar = OCP\Share::getItemSharedWithBySource('calendar', $id);
-			if (!$sharedCalendar || !($sharedCalendar['permissions'] & OCP\Share::PERMISSION_CREATE)) {
+			if (!$sharedCalendar || !($sharedCalendar['permissions'] & OCP\PERMISSION_CREATE)) {
 				throw new Sabre_DAV_Exception_Forbidden(
 					OC_Calendar_App::$l10n->t(
 						'You do not have the permissions to add events to this calendar.'
@@ -189,9 +186,11 @@ class OC_Calendar_Object{
 		$oldobject = self::find($id);
 
 		$calendar = OC_Calendar_Calendar::find($oldobject['calendarid']);
+		$oldvobject = OC_VObject::parse($oldobject['calendardata']);
 		if ($calendar['userid'] != OCP\User::getUser()) {
-			$sharedCalendar = OCP\Share::getItemSharedWithBySource('calendar', $oldobject['calendarid']);
-			if (!$sharedCalendar || !($sharedCalendar['permissions'] & OCP\Share::PERMISSION_UPDATE)) {
+			$sharedCalendar = OCP\Share::getItemSharedWithBySource('calendar', $id);
+			$sharedAccessClassPermissions = OC_Calendar_App::getAccessClassPermissions($oldvobject->VEVENT->CLASS->value);
+			if (!$sharedCalendar || !($sharedCalendar['permissions'] & OCP\PERMISSION_UPDATE) || !($sharedAccessClassPermissions & OCP\PERMISSION_UPDATE)) {
 				throw new Exception(
 					OC_Calendar_App::$l10n->t(
 						'You do not have the permissions to edit this event.'
@@ -200,7 +199,7 @@ class OC_Calendar_Object{
 			}
 		}
 		$object = OC_VObject::parse($data);
-		OC_Calendar_App::loadCategoriesFromVCalendar($object);
+		OC_Calendar_App::loadCategoriesFromVCalendar($id, $object);
 		list($type,$startdate,$enddate,$summary,$repeating,$uid) = self::extractData($object);
 
 		$stmt = OCP\DB::prepare( 'UPDATE `*PREFIX*calendar_objects` SET `objecttype`=?,`startdate`=?,`enddate`=?,`repeating`=?,`summary`=?,`calendardata`=?,`lastmodified`= ? WHERE `id` = ?' );
@@ -223,9 +222,11 @@ class OC_Calendar_Object{
 		$oldobject = self::findWhereDAVDataIs($cid,$uri);
 
 		$calendar = OC_Calendar_Calendar::find($cid);
+		$oldvobject = OC_VObject::parse($oldobject['calendardata']);
 		if ($calendar['userid'] != OCP\User::getUser()) {
 			$sharedCalendar = OCP\Share::getItemSharedWithBySource('calendar', $cid);
-			if (!$sharedCalendar || !($sharedCalendar['permissions'] & OCP\Share::PERMISSION_UPDATE)) {
+			$sharedAccessClassPermissions = OC_Calendar_App::getAccessClassPermissions($oldvobject->VEVENT->CLASS->value);
+			if (!$sharedCalendar || !($sharedCalendar['permissions'] & OCP\PERMISSION_UPDATE) || !($sharedAccessClassPermissions & OCP\PERMISSION_UPDATE)) {
 				throw new Sabre_DAV_Exception_Forbidden(
 					OC_Calendar_App::$l10n->t(
 						'You do not have the permissions to edit this event.'
@@ -253,9 +254,11 @@ class OC_Calendar_Object{
 	public static function delete($id) {
 		$oldobject = self::find($id);
 		$calendar = OC_Calendar_Calendar::find($oldobject['calendarid']);
+		$object = OC_VObject::parse($oldobject['calendardata']);
 		if ($calendar['userid'] != OCP\User::getUser()) {
-			$sharedCalendar = OCP\Share::getItemSharedWithBySource('calendar', $oldobject['calendarid']);
-			if (!$sharedCalendar || !($sharedCalendar['permissions'] & OCP\Share::PERMISSION_DELETE)) {
+			$sharedCalendar = OCP\Share::getItemSharedWithBySource('calendar', $id);
+			$sharedAccessClassPermissions = OC_Calendar_App::getAccessClassPermissions($object->VEVENT->CLASS->value);
+			if (!$sharedCalendar || !($sharedCalendar['permissions'] & OCP\PERMISSION_DELETE) || !($sharedAccessClassPermissions & OCP\PERMISSION_DELETE)) {
 				throw new Exception(
 					OC_Calendar_App::$l10n->t(
 						'You do not have the permissions to delete this event.'
@@ -271,6 +274,8 @@ class OC_Calendar_Object{
 
 		OCP\Util::emitHook('OC_Calendar', 'deleteEvent', $id);
 
+		OC_Calendar_App::getVCategories()->purgeObject($id);
+
 		return true;
 	}
 
@@ -285,7 +290,7 @@ class OC_Calendar_Object{
 		$calendar = OC_Calendar_Calendar::find($cid);
 		if ($calendar['userid'] != OCP\User::getUser()) {
 			$sharedCalendar = OCP\Share::getItemSharedWithBySource('calendar', $cid);
-			if (!$sharedCalendar || !($sharedCalendar['permissions'] & OCP\Share::PERMISSION_DELETE)) {
+			if (!$sharedCalendar || !($sharedCalendar['permissions'] & OCP\PERMISSION_DELETE)) {
 				throw new Sabre_DAV_Exception_Forbidden(
 					OC_Calendar_App::$l10n->t(
 						'You do not have the permissions to delete this event.'
@@ -304,8 +309,8 @@ class OC_Calendar_Object{
 	public static function moveToCalendar($id, $calendarid) {
 		$calendar = OC_Calendar_Calendar::find($calendarid);
 		if ($calendar['userid'] != OCP\User::getUser()) {
-			$sharedCalendar = OCP\Share::getItemSharedWithBySource('calendar', $calendarid);
-			if (!$sharedCalendar || !($sharedCalendar['permissions'] & OCP\Share::PERMISSION_CREATE)) {
+			$sharedCalendar = OCP\Share::getItemSharedWithBySource('calendar', $id);
+			if (!$sharedCalendar || !($sharedCalendar['permissions'] & OCP\PERMISSION_DELETE)) {
 				throw new Exception(
 					OC_Calendar_App::$l10n->t(
 						'You do not have the permissions to add events to this calendar.'
@@ -414,7 +419,7 @@ class OC_Calendar_Object{
 	 * Furthermore it converts the time to UTC.
 	 */
 	public static function getUTCforMDB($datetime) {
-		return date('Y-m-d H:i', $datetime->format('U') - $datetime->getOffset());
+		return date('Y-m-d H:i', $datetime->format('U'));
 	}
 
 	/**
@@ -447,6 +452,55 @@ class OC_Calendar_Object{
 		return $dtend;
 	}
 
+	/**
+	 * @brief Remove all properties which should not be exported for the AccessClass Confidential
+	 * @param string $calendarId Calendar ID
+	 * @param Sabre_VObject $vobject Sabre VObject
+	 * @return object
+	 */
+	public static function cleanByAccessClass($calendarId, $vobject) {
+
+		// Do not clean your own calendar
+		if(OC_Calendar_Object::getowner($calendarId) === OCP\USER::getUser()) {
+			return $vobject;
+		}
+
+		$vevent = $vobject->VEVENT;
+		if(!is_null($vevent->CLASS) && $vevent->CLASS->value == 'CONFIDENTIAL') {
+			foreach ($vevent->children as &$property) {
+				switch($property->name) {
+					case 'CREATED':
+					case 'DTSTART':
+					case 'RRULE':
+					case 'DURATION':
+					case 'DTEND':
+					case 'CLASS':
+					case 'UID':
+						break;
+					case 'SUMMARY':
+						$property->value = OC_Calendar_App::$l10n->t('Busy');
+						break;
+					default:
+						$vevent->__unset($property->name);
+						break;
+				}
+			}
+		}
+		return $vobject;
+	}
+
+	/**
+	 * @brief returns the options for the access class of an event
+	 * @return array - valid inputs for the access class of an event
+	 */
+	public static function getAccessClassOptions($l10n) {
+		return array(
+			'PUBLIC'       => (string)$l10n->t('Public'),
+			'PRIVATE'      => (string)$l10n->t('Private'),
+			'CONFIDENTIAL' => (string)$l10n->t('Confidential')
+		);
+	}
+
 	/**
 	 * @brief returns the options for the repeat rule of an repeating event
 	 * @return array - valid inputs for the repeat rule of an repeating event
@@ -764,7 +818,7 @@ class OC_Calendar_Object{
 		$vevent = new OC_VObject('VEVENT');
 		$vcalendar->add($vevent);
 
-		$vevent->setDateTime('CREATED', 'now', Sabre_VObject_Property_DateTime::UTC);
+		$vevent->setDateTime('CREATED', 'now', Sabre\VObject\Property\DateTime::UTC);
 
 		$vevent->setUID();
 		return self::updateVCalendarFromRequest($request, $vcalendar);
@@ -777,6 +831,7 @@ class OC_Calendar_Object{
 	 * @return object updated $vcalendar
 	 */
 	public static function updateVCalendarFromRequest($request, $vcalendar) {
+		$accessclass = $request["accessclass"];
 		$title = $request["title"];
 		$location = $request["location"];
 		$categories = $request["categories"];
@@ -933,25 +988,28 @@ class OC_Calendar_Object{
 		}
 
 
-		$vevent->setDateTime('LAST-MODIFIED', 'now', Sabre_VObject_Property_DateTime::UTC);
-		$vevent->setDateTime('DTSTAMP', 'now', Sabre_VObject_Property_DateTime::UTC);
+		$vevent->setDateTime('LAST-MODIFIED', 'now', Sabre\VObject\Property\DateTime::UTC);
+		$vevent->setDateTime('DTSTAMP', 'now', Sabre\VObject\Property\DateTime::UTC);
 		$vevent->setString('SUMMARY', $title);
 
 		if($allday) {
 			$start = new DateTime($from);
 			$end = new DateTime($to.' +1 day');
-			$vevent->setDateTime('DTSTART', $start, Sabre_VObject_Property_DateTime::DATE);
-			$vevent->setDateTime('DTEND', $end, Sabre_VObject_Property_DateTime::DATE);
+			$vevent->setDateTime('DTSTART', $start, Sabre\VObject\Property\DateTime::DATE);
+			$vevent->setDateTime('DTEND', $end, Sabre\VObject\Property\DateTime::DATE);
 		}else{
 			$timezone = OC_Calendar_App::getTimezone();
 			$timezone = new DateTimeZone($timezone);
 			$start = new DateTime($from.' '.$fromtime, $timezone);
+			$start->setTimezone(new DateTimeZone('UTC'));
 			$end = new DateTime($to.' '.$totime, $timezone);
-			$vevent->setDateTime('DTSTART', $start, Sabre_VObject_Property_DateTime::LOCALTZ);
-			$vevent->setDateTime('DTEND', $end, Sabre_VObject_Property_DateTime::LOCALTZ);
+			$end->setTimezone(new DateTimeZone('UTC'));
+			$vevent->setDateTime('DTSTART', $start, Sabre\VObject\Property\DateTime::UTC);
+			$vevent->setDateTime('DTEND', $end, Sabre\VObject\Property\DateTime::UTC);
 		}
 		unset($vevent->DURATION);
 
+		$vevent->setString('CLASS', $accessclass);
 		$vevent->setString('LOCATION', $location);
 		$vevent->setString('DESCRIPTION', $description);
 		$vevent->setString('CATEGORIES', $categories);
@@ -971,7 +1029,11 @@ class OC_Calendar_Object{
 	public static function getowner($id) {
 		$event = self::find($id);
 		$cal = OC_Calendar_Calendar::find($event['calendarid']);
-		return $cal['userid'];
+		if(array_key_exists('userid', $cal)){
+			return $cal['userid'];
+		}else{
+			return null;
+		}
 	}
 
 	/**
