@@ -6,17 +6,20 @@
  * See the COPYING-README file.
  */
 
-class Test_Filestorage_DAV extends Test_FileStorage {
+namespace Test\Files\Storage;
+
+class DAV extends Storage {
+
 	private $config;
 
 	public function setUp() {
 		$id = uniqid();
 		$this->config = include('files_external/tests/config.php');
-		if (!is_array($this->config) or !isset($this->config['webdav']) or !$this->config['webdav']['run']) {
+		if ( ! is_array($this->config) or ! isset($this->config['webdav']) or ! $this->config['webdav']['run']) {
 			$this->markTestSkipped('WebDAV backend not configured');
 		}
 		$this->config['webdav']['root'] .= '/' . $id; //make sure we have an new empty folder to work in
-		$this->instance = new OC_Filestorage_DAV($this->config['webdav']);
+		$this->instance = new \OC\Files\Storage\DAV($this->config['webdav']);
 	}
 
 	public function tearDown() {
