@@ -36,6 +36,16 @@ class OC_App{
 	static private $appTypes = array();
 	static private $loadedApps = array();
 	static private $checkedApps = array();
+	static private $altLogin = array();
+
+	/**
+	 * @brief clean the appid
+	 * @param $app Appid that needs to be cleaned
+	 * @return string
+	 */
+	public static function cleanAppId($app) {
+		return str_replace(array('\0', '/', '\\', '..'), '', $app);
+	}
 
 	/**
 	 * @brief loads all apps
@@ -63,8 +73,8 @@ class OC_App{
 
 		if (!defined('DEBUG') || !DEBUG) {
 			if (is_null($types)
-			    && empty(OC_Util::$core_scripts) 
-			    && empty(OC_Util::$core_styles)) {
+				&& empty(OC_Util::$core_scripts)
+				&& empty(OC_Util::$core_styles)) {
 				OC_Util::$core_scripts = OC_Util::$scripts;
 				OC_Util::$scripts = array();
 				OC_Util::$core_styles = OC_Util::$styles;
@@ -92,7 +102,7 @@ class OC_App{
 	 * @param string/array $types
 	 * @return bool
 	 */
-	public static function isType($app,$types) {
+	public static function isType($app, $types) {
 		if(is_string($types)) {
 			$types=array($types);
 		}
@@ -138,14 +148,32 @@ class OC_App{
 		OC_Appconfig::setValue($app, 'types', $appTypes);
 	}
 
+	/**
+	 * check if app is shipped
+	 * @param string $appid the id of the app to check
+	 * @return bool
+	 *
+	 * Check if an app that is installed is a shipped app or installed from the appstore.
+	 */
+	public static function isShipped($appid){
+		$info = self::getAppInfo($appid);
+		if(isset($info['shipped']) && $info['shipped']=='true') {
+			return true;
+		} else {
+			return false;
+		}
+	}
+
 	/**
 	 * get all enabled apps
 	 */
 	public static function getEnabledApps() {
-		if(!OC_Config::getValue('installed', false))
+		if(!OC_Config::getValue('installed', false)) {
 			return array();
+		}
 		$apps=array('files');
-		$query = OC_DB::prepare( 'SELECT `appid` FROM `*PREFIX*appconfig` WHERE `configkey` = \'enabled\' AND `configvalue`=\'yes\'' );
+		$query = OC_DB::prepare( 'SELECT `appid` FROM `*PREFIX*appconfig`'
+			.' WHERE `configkey` = \'enabled\' AND `configvalue`=\'yes\'' );
 		$result=$query->execute();
 		while($row=$result->fetchRow()) {
 			if(array_search($row['appid'], $apps)===false) {
@@ -163,7 +191,7 @@ class OC_App{
 	 * This function checks whether or not an app is enabled.
 	 */
 	public static function isEnabled( $app ) {
-		if( 'files'==$app or 'yes' == OC_Appconfig::getValue( $app, 'enabled' )) {
+		if( 'files'==$app or ('yes' == OC_Appconfig::getValue( $app, 'enabled' ))) {
 			return true;
 		}
 
@@ -183,9 +211,11 @@ class OC_App{
 			if(!is_numeric($app)) {
 				$app = OC_Installer::installShippedApp($app);
 			}else{
+				$appdata=OC_OCSClient::getApplication($app);
 				$download=OC_OCSClient::getApplicationDownload($app, 1);
 				if(isset($download['downloadlink']) and $download['downloadlink']!='') {
-					$app=OC_Installer::installApp(array('source'=>'http','href'=>$download['downloadlink']));
+					$info = array('source'=>'http', 'href'=>$download['downloadlink'], 'appdata'=>$appdata);
+					$app=OC_Installer::installApp($info);
 				}
 			}
 		}
@@ -193,11 +223,17 @@ class OC_App{
 			// check if the app is compatible with this version of ownCloud
 			$info=OC_App::getAppInfo($app);
 			$version=OC_Util::getVersion();
-			if(!isset($info['require']) or ($version[0]>$info['require'])) {
-				OC_Log::write('core', 'App "'.$info['name'].'" can\'t be installed because it is not compatible with this version of ownCloud', OC_Log::ERROR);
+			if(!isset($info['require']) or !self::isAppVersionCompatible($version, $info['require'])) {
+				OC_Log::write('core',
+					'App "'.$info['name'].'" can\'t be installed because it is'
+					.' not compatible with this version of ownCloud',
+					OC_Log::ERROR);
 				return false;
 			}else{
 				OC_Appconfig::setValue( $app, 'enabled', 'yes' );
+				if(isset($appdata['id'])) {
+					OC_Appconfig::setValue( $app, 'ocsid', $appdata['id'] );
+				}
 				return true;
 			}
 		}else{
@@ -213,8 +249,13 @@ class OC_App{
 	 * This function set an app as disabled in appconfig.
 	 */
 	public static function disable( $app ) {
-		// check if app is a shiped app or not. if not delete
+		// check if app is a shipped app or not. if not delete
 		OC_Appconfig::setValue( $app, 'enabled', 'no' );
+
+		// check if app is a shipped app or not. if not delete
+		if(!OC_App::isShipped( $app )) {
+			OC_Installer::removeApp( $app );
+		}
 	}
 
 	/**
@@ -253,10 +294,29 @@ class OC_App{
 	 * highlighting the current position of the user.
 	 */
 	public static function setActiveNavigationEntry( $id ) {
+		// load all the apps, to make sure we have all the navigation entries
+		self::loadApps();
 		self::$activeapp = $id;
 		return true;
 	}
 
+	/**
+	 * @brief Get the navigation entries for the $app
+	 * @param string $app app
+	 * @return array of the $data added with addNavigationEntry
+	 */
+	public static function getAppNavigationEntries($app) {
+		if(is_file(self::getAppPath($app).'/appinfo/app.php')) {
+			$save = self::$navigation;
+			self::$navigation = array();
+			require $app.'/appinfo/app.php';
+			$app_entries = self::$navigation;
+			self::$navigation = $save;
+			return $app_entries;
+		}
+		return array();
+	}
+
 	/**
 	 * @brief gets the active Menu entry
 	 * @return string id or empty string
@@ -282,33 +342,70 @@ class OC_App{
 		// by default, settings only contain the help menu
 		if(OC_Config::getValue('knowledgebaseenabled', true)==true) {
 			$settings = array(
-				array( "id" => "help", "order" => 1000, "href" => OC_Helper::linkTo( "settings", "help.php" ), "name" => $l->t("Help"), "icon" => OC_Helper::imagePath( "settings", "help.svg" ))
+				array(
+					"id" => "help",
+					"order" => 1000,
+					"href" => OC_Helper::linkToRoute( "settings_help" ),
+					"name" => $l->t("Help"),
+					"icon" => OC_Helper::imagePath( "settings", "help.svg" )
+				)
 			);
 		}
 
 		// if the user is logged-in
 		if (OC_User::isLoggedIn()) {
 			// personal menu
-			$settings[] = array( "id" => "personal", "order" => 1, "href" => OC_Helper::linkTo( "settings", "personal.php" ), "name" => $l->t("Personal"), "icon" => OC_Helper::imagePath( "settings", "personal.svg" ));
+			$settings[] = array(
+				"id" => "personal",
+				"order" => 1,
+				"href" => OC_Helper::linkToRoute( "settings_personal" ),
+				"name" => $l->t("Personal"),
+				"icon" => OC_Helper::imagePath( "settings", "personal.svg" )
+			);
 
 			// if there are some settings forms
-			if(!empty(self::$settingsForms))
+			if(!empty(self::$settingsForms)) {
 				// settings menu
-				$settings[]=array( "id" => "settings", "order" => 1000, "href" => OC_Helper::linkTo( "settings", "settings.php" ), "name" => $l->t("Settings"), "icon" => OC_Helper::imagePath( "settings", "settings.svg" ));
+				$settings[]=array(
+					"id" => "settings",
+					"order" => 1000,
+					"href" => OC_Helper::linkToRoute( "settings_settings" ),
+					"name" => $l->t("Settings"),
+					"icon" => OC_Helper::imagePath( "settings", "settings.svg" )
+				);
+			}
 
 			//SubAdmins are also allowed to access user management
-			if(OC_SubAdmin::isSubAdmin($_SESSION["user_id"]) || OC_Group::inGroup( $_SESSION["user_id"], "admin" )) {
+			if(OC_SubAdmin::isSubAdmin(OC_User::getUser())) {
 				// admin users menu
-				$settings[] = array( "id" => "core_users", "order" => 2, "href" => OC_Helper::linkTo( "settings", "users.php" ), "name" => $l->t("Users"), "icon" => OC_Helper::imagePath( "settings", "users.svg" ));
+				$settings[] = array(
+					"id" => "core_users",
+					"order" => 2,
+					"href" => OC_Helper::linkToRoute( "settings_users" ),
+					"name" => $l->t("Users"),
+					"icon" => OC_Helper::imagePath( "settings", "users.svg" )
+				);
 			}
 
 
 			// if the user is an admin
-			if(OC_Group::inGroup( $_SESSION["user_id"], "admin" )) {
+			if(OC_User::isAdminUser(OC_User::getUser())) {
 				// admin apps menu
-				$settings[] = array( "id" => "core_apps", "order" => 3, "href" => OC_Helper::linkTo( "settings", "apps.php" ).'?installed', "name" => $l->t("Apps"), "icon" => OC_Helper::imagePath( "settings", "apps.svg" ));
+				$settings[] = array(
+					"id" => "core_apps",
+					"order" => 3,
+					"href" => OC_Helper::linkToRoute( "settings_apps" ).'?installed',
+					"name" => $l->t("Apps"),
+					"icon" => OC_Helper::imagePath( "settings", "apps.svg" )
+				);
 
-				$settings[]=array( "id" => "admin", "order" => 1000, "href" => OC_Helper::linkTo( "settings", "admin.php" ), "name" => $l->t("Admin"), "icon" => OC_Helper::imagePath( "settings", "admin.svg" ));
+				$settings[]=array(
+					"id" => "admin",
+					"order" => 1000,
+					"href" => OC_Helper::linkToRoute( "settings_admin" ),
+					"name" => $l->t("Admin"),
+					"icon" => OC_Helper::imagePath( "settings", "admin.svg" )
+				);
 			}
 		}
 
@@ -319,7 +416,6 @@ class OC_App{
 	/// This is private as well. It simply works, so don't ask for more details
 	private static function proceedNavigation( $list ) {
 		foreach( $list as &$naventry ) {
-			$naventry['subnavigation'] = array();
 			if( $naventry['id'] == self::$activeapp ) {
 				$naventry['active'] = true;
 			}
@@ -342,8 +438,9 @@ class OC_App{
 		}
 
 		foreach(OC::$APPSROOTS as $dir) {
-			if(isset($dir['writable']) && $dir['writable']===true)
+			if(isset($dir['writable']) && $dir['writable']===true) {
 				return $dir['path'];
+			}
 		}
 
 		OC_Log::write('core', 'No application directories are marked as writable.', OC_Log::ERROR);
@@ -390,9 +487,8 @@ class OC_App{
 	 */
 	public static function getAppVersion($appid) {
 		$file= self::getAppPath($appid).'/appinfo/version';
-		$version=@file_get_contents($file);
-		if($version) {
-			return trim($version);
+		if(is_file($file) && $version = trim(file_get_contents($file))) {
+			return $version;
 		}else{
 			$appData=self::getAppInfo($appid);
 			return isset($appData['version'])? $appData['version'] : '';
@@ -406,7 +502,7 @@ class OC_App{
 	 * @return array
 	 * @note all data is read from info.xml, not just pre-defined fields
 	*/
-	public static function getAppInfo($appid,$path=false) {
+	public static function getAppInfo($appid, $path=false) {
 		if($path) {
 			$file=$appid;
 		}else{
@@ -458,7 +554,7 @@ class OC_App{
 			}
 		}
 		self::$appInfo[$appid]=$data;
-		
+
 		return $data;
 	}
 
@@ -470,8 +566,6 @@ class OC_App{
 	 * entries are sorted by the key 'order' ascending. Additional to the keys
 	 * given for each app the following keys exist:
 	 *   - active: boolean, signals if the user is on this navigation entry
-	 *   - children: array that is empty if the key 'active' is false or
-	 *     contains the subentries if the key 'active' is true
 	 */
 	public static function getNavigation() {
 		$navigation = self::proceedNavigation( self::$navigation );
@@ -483,8 +577,14 @@ class OC_App{
 	 * @return string
 	 */
 	public static function getCurrentApp() {
-		$script=substr($_SERVER["SCRIPT_NAME"], strlen(OC::$WEBROOT)+1);
+		$script=substr(OC_Request::scriptName(), strlen(OC::$WEBROOT)+1);
 		$topFolder=substr($script, 0, strpos($script, '/'));
+		if (empty($topFolder)) {
+			$path_info = OC_Request::getPathInfo();
+			if ($path_info) {
+				$topFolder=substr($path_info, 1, strpos($path_info, '/', 1)-1);
+			}
+		}
 		if($topFolder=='apps') {
 			$length=strlen($topFolder);
 			return substr($script, $length+1, strpos($script, '/', $length+1)-$length-1);
@@ -521,70 +621,147 @@ class OC_App{
 	/**
 	 * register a settings form to be shown
 	 */
-	public static function registerSettings($app,$page) {
+	public static function registerSettings($app, $page) {
 		self::$settingsForms[]= $app.'/'.$page.'.php';
 	}
 
 	/**
 	 * register an admin form to be shown
 	 */
-	public static function registerAdmin($app,$page) {
+	public static function registerAdmin($app, $page) {
 		self::$adminForms[]= $app.'/'.$page.'.php';
 	}
 
 	/**
 	 * register a personal form to be shown
 	 */
-	public static function registerPersonal($app,$page) {
+	public static function registerPersonal($app, $page) {
 		self::$personalForms[]= $app.'/'.$page.'.php';
 	}
 
+	public static function registerLogIn($entry) {
+		self::$altLogin[] = $entry;
+	}
+
+	public static function getAlternativeLogIns() {
+		return self::$altLogin;
+	}
+
 	/**
 	 * @brief: get a list of all apps in the apps folder
 	 * @return array or app names (string IDs)
 	 * @todo: change the name of this method to getInstalledApps, which is more accurate
 	 */
 	public static function getAllApps() {
-	
+
 		$apps=array();
-		
+
 		foreach ( OC::$APPSROOTS as $apps_dir ) {
 			if(! is_readable($apps_dir['path'])) {
-				OC_Log::write('core', 'unable to read app folder : ' .$apps_dir['path'] , OC_Log::WARN);
+				OC_Log::write('core', 'unable to read app folder : ' .$apps_dir['path'], OC_Log::WARN);
 				continue;
 			}
 			$dh = opendir( $apps_dir['path'] );
-			
+
 			while( $file = readdir( $dh ) ) {
-			
-				if ( 
-				$file[0] != '.' 
-				and is_file($apps_dir['path'].'/'.$file.'/appinfo/app.php' ) 
-				) {
-				
+
+				if ($file[0] != '.' and is_file($apps_dir['path'].'/'.$file.'/appinfo/app.php')) {
+
 					$apps[] = $file;
-					
+
 				}
-				
+
 			}
-			
+
 		}
-		
+
 		return $apps;
 	}
-	
+
+	/**
+	 * @brief: Lists all apps, this is used in apps.php
+	 * @return array
+	 */
+	public static function listAllApps() {
+		$installedApps = OC_App::getAllApps();
+
+		//TODO which apps do we want to blacklist and how do we integrate
+		// blacklisting with the multi apps folder feature?
+
+		$blacklist = array('files');//we dont want to show configuration for these
+		$appList = array();
+
+		foreach ( $installedApps as $app ) {
+			if ( array_search( $app, $blacklist ) === false ) {
+
+				$info=OC_App::getAppInfo($app);
+
+				if (!isset($info['name'])) {
+					OC_Log::write('core', 'App id "'.$app.'" has no name in appinfo', OC_Log::ERROR);
+					continue;
+				}
+
+				if ( OC_Appconfig::getValue( $app, 'enabled', 'no') == 'yes' ) {
+					$active = true;
+				} else {
+					$active = false;
+				}
+
+				$info['active'] = $active;
+
+				if(isset($info['shipped']) and ($info['shipped']=='true')) {
+					$info['internal']=true;
+					$info['internallabel']='Internal App';
+					$info['internalclass']='';
+					$info['update']=false;
+				} else {
+					$info['internal']=false;
+					$info['internallabel']='3rd Party';
+					$info['internalclass']='externalapp';
+					$info['update']=OC_Installer::isUpdateAvailable($app);
+				}
+
+				$info['preview'] = OC_Helper::imagePath('settings', 'trans.png');
+				$info['version'] = OC_App::getAppVersion($app);
+				$appList[] = $info;
+			}
+		}
+		$remoteApps = OC_App::getAppstoreApps();
+		if ( $remoteApps ) {
+			// Remove duplicates
+			foreach ( $appList as $app ) {
+				foreach ( $remoteApps AS $key => $remote ) {
+					if (
+						$app['name'] == $remote['name']
+						// To set duplicate detection to use OCS ID instead of string name,
+						// enable this code, remove the line of code above,
+						// and add <ocs_id>[ID]</ocs_id> to info.xml of each 3rd party app:
+						// OR $app['ocs_id'] == $remote['ocs_id']
+						) {
+						unset( $remoteApps[$key]);
+					}
+				}
+			}
+			$combinedApps = array_merge( $appList, $remoteApps );
+		} else {
+			$combinedApps = $appList;
+		}
+		return $combinedApps;
+	}
+
 	/**
 	 * @brief: get a list of all apps on apps.owncloud.com
-	 * @return array, multi-dimensional array of apps. Keys: id, name, type, typename, personid, license, detailpage, preview, changed, description
+	 * @return array, multi-dimensional array of apps.
+	 *     Keys: id, name, type, typename, personid, license, detailpage, preview, changed, description
 	 */
 	public static function getAppstoreApps( $filter = 'approved' ) {
-		$catagoryNames = OC_OCSClient::getCategories();
-		if ( is_array( $catagoryNames ) ) {
+		$categoryNames = OC_OCSClient::getCategories();
+		if ( is_array( $categoryNames ) ) {
 			// Check that categories of apps were retrieved correctly
-			if ( ! $categories = array_keys( $catagoryNames ) ) {
+			if ( ! $categories = array_keys( $categoryNames ) ) {
 				return false;
 			}
-			
+
 			$page = 0;
 			$remoteApps = OC_OCSClient::getApplications( $categories, $page, $filter );
 			$app1 = array();
@@ -594,18 +771,27 @@ class OC_App{
 				$app1[$i]['author'] = $app['personid'];
 				$app1[$i]['ocs_id'] = $app['id'];
 				$app1[$i]['internal'] = $app1[$i]['active'] = 0;
+				$app1[$i]['update'] = false;
+				if($app['label']=='recommended') {
+					$app1[$i]['internallabel'] = 'Recommended';
+					$app1[$i]['internalclass'] = 'recommendedapp';
+				}else{
+					$app1[$i]['internallabel'] = '3rd Party';
+					$app1[$i]['internalclass'] = 'externalapp';
+				}
+
 
 				// rating img
-				if($app['score']>=0     and $app['score']<5) 	$img=OC_Helper::imagePath( "core", "rating/s1.png" );
-				elseif($app['score']>=5 and $app['score']<15) 	$img=OC_Helper::imagePath( "core", "rating/s2.png" );
-				elseif($app['score']>=15 and $app['score']<25) 	$img=OC_Helper::imagePath( "core", "rating/s3.png" );
-				elseif($app['score']>=25 and $app['score']<35) 	$img=OC_Helper::imagePath( "core", "rating/s4.png" );
-				elseif($app['score']>=35 and $app['score']<45) 	$img=OC_Helper::imagePath( "core", "rating/s5.png" );
-				elseif($app['score']>=45 and $app['score']<55) 	$img=OC_Helper::imagePath( "core", "rating/s6.png" );
-				elseif($app['score']>=55 and $app['score']<65) 	$img=OC_Helper::imagePath( "core", "rating/s7.png" );
-				elseif($app['score']>=65 and $app['score']<75) 	$img=OC_Helper::imagePath( "core", "rating/s8.png" );
-				elseif($app['score']>=75 and $app['score']<85) 	$img=OC_Helper::imagePath( "core", "rating/s9.png" );
-				elseif($app['score']>=85 and $app['score']<95) 	$img=OC_Helper::imagePath( "core", "rating/s10.png" );
+				if($app['score']>=0     and $app['score']<5)	$img=OC_Helper::imagePath( "core", "rating/s1.png" );
+				elseif($app['score']>=5 and $app['score']<15)	$img=OC_Helper::imagePath( "core", "rating/s2.png" );
+				elseif($app['score']>=15 and $app['score']<25)	$img=OC_Helper::imagePath( "core", "rating/s3.png" );
+				elseif($app['score']>=25 and $app['score']<35)	$img=OC_Helper::imagePath( "core", "rating/s4.png" );
+				elseif($app['score']>=35 and $app['score']<45)	$img=OC_Helper::imagePath( "core", "rating/s5.png" );
+				elseif($app['score']>=45 and $app['score']<55)	$img=OC_Helper::imagePath( "core", "rating/s6.png" );
+				elseif($app['score']>=55 and $app['score']<65)	$img=OC_Helper::imagePath( "core", "rating/s7.png" );
+				elseif($app['score']>=65 and $app['score']<75)	$img=OC_Helper::imagePath( "core", "rating/s8.png" );
+				elseif($app['score']>=75 and $app['score']<85)	$img=OC_Helper::imagePath( "core", "rating/s9.png" );
+				elseif($app['score']>=85 and $app['score']<95)	$img=OC_Helper::imagePath( "core", "rating/s10.png" );
 				elseif($app['score']>=95 and $app['score']<100)	$img=OC_Helper::imagePath( "core", "rating/s11.png" );
 
 				$app1[$i]['score'] = '<img src="'.$img.'"> Score: '.$app['score'].'%';
@@ -633,12 +819,17 @@ class OC_App{
 		if ($currentVersion) {
 			$installedVersion = $versions[$app];
 			if (version_compare($currentVersion, $installedVersion, '>')) {
-				OC_Log::write($app, 'starting app upgrade from '.$installedVersion.' to '.$currentVersion, OC_Log::DEBUG);
+				$info = self::getAppInfo($app);
+				OC_Log::write($app,
+					'starting app upgrade from '.$installedVersion.' to '.$currentVersion,
+					OC_Log::DEBUG);
 				try {
 					OC_App::updateApp($app);
+					OC_Hook::emit('update', 'success', 'Updated '.$info['name'].' app');
 				}
 				catch (Exception $e) {
 					echo 'Failed to upgrade "'.$app.'". Exception="'.$e->getMessage().'"';
+					OC_Hook::emit('update', 'failure', 'Failed to update '.$info['name'].' app: '.$e->getMessage());
 					die;
 				}
 				OC_Appconfig::setValue($app, 'installed_version', OC_App::getAppVersion($app));
@@ -659,14 +850,50 @@ class OC_App{
 		$version = OC_Util::getVersion();
 		foreach($apps as $app) {
 			// check if the app is compatible with this version of ownCloud
-			$info = OC_App::getAppInfo($app);			
-			if(!isset($info['require']) or (($version[0].'.'.$version[1])>$info['require'])) {
-				OC_Log::write('core', 'App "'.$info['name'].'" ('.$app.') can\'t be used because it is not compatible with this version of ownCloud', OC_Log::ERROR);
+			$info = OC_App::getAppInfo($app);
+			if(!isset($info['require']) or !self::isAppVersionCompatible($version, $info['require'])) {
+				OC_Log::write('core',
+					'App "'.$info['name'].'" ('.$app.') can\'t be used because it is'
+					.' not compatible with this version of ownCloud',
+					OC_Log::ERROR);
 				OC_App::disable( $app );
+				OC_Hook::emit('update', 'success', 'Disabled '.$info['name'].' app because it is not compatible');
 			}
 		}
 	}
 
+
+	/**
+	 * Compares the app version with the owncloud version to see if the app 
+	 * requires a newer version than the currently active one
+	 * @param array $owncloudVersions array with 3 entries: major minor bugfix
+	 * @param string $appRequired the required version from the xml 
+	 * major.minor.bugfix
+	 * @return boolean true if compatible, otherwise false
+	 */
+	public static function isAppVersionCompatible($owncloudVersions, $appRequired){
+		$appVersions = explode('.', $appRequired);
+
+		for($i=0; $i<count($appVersions); $i++){
+			$appVersion = (int) $appVersions[$i];
+
+			if(isset($owncloudVersions[$i])){
+				$owncloudVersion = $owncloudVersions[$i];
+			} else {
+				$owncloudVersion = 0;
+			}
+
+			if($owncloudVersion < $appVersion){
+				return false;
+			} elseif ($owncloudVersion > $appVersion) {
+				return true;
+			}
+		}
+
+		return true;
+	}
+
+
 	/**
 	 * get the installed version of all apps
 	 */
@@ -676,7 +903,8 @@ class OC_App{
 			return $versions; // when function is used besides in checkUpgrade
 		}
 		$versions=array();
-		$query = OC_DB::prepare( 'SELECT `appid`, `configvalue` FROM `*PREFIX*appconfig` WHERE `configkey` = \'installed_version\'' );
+		$query = OC_DB::prepare( 'SELECT `appid`, `configvalue` FROM `*PREFIX*appconfig`'
+			.' WHERE `configkey` = \'installed_version\'' );
 		$result = $query->execute();
 		while($row = $result->fetchRow()) {
 			$versions[$row['appid']]=$row['configvalue'];
@@ -689,6 +917,10 @@ class OC_App{
 	 * @param string $appid
 	 */
 	public static function updateApp($appid) {
+		if(file_exists(self::getAppPath($appid).'/appinfo/preupdate.php')) {
+			self::loadApp($appid);
+			include self::getAppPath($appid).'/appinfo/preupdate.php';
+		}
 		if(file_exists(self::getAppPath($appid).'/appinfo/database.xml')) {
 			OC_DB::updateDbFromStructure(self::getAppPath($appid).'/appinfo/database.xml');
 		}
@@ -714,18 +946,18 @@ class OC_App{
 
 	/**
 	 * @param string $appid
-	 * @return OC_FilesystemView
+	 * @return \OC\Files\View
 	 */
 	public static function getStorage($appid) {
 		if(OC_App::isEnabled($appid)) {//sanity check
 			if(OC_User::isLoggedIn()) {
-				$view = new OC_FilesystemView('/'.OC_User::getUser());
+				$view = new \OC\Files\View('/'.OC_User::getUser());
 				if(!$view->file_exists($appid)) {
 					$view->mkdir($appid);
 				}
-				return new OC_FilesystemView('/'.OC_User::getUser().'/'.$appid);
+				return new \OC\Files\View('/'.OC_User::getUser().'/'.$appid);
 			}else{
-				OC_Log::write('core', 'Can\'t get app storage, app, user not logged in', OC_Log::ERROR);
+				OC_Log::write('core', 'Can\'t get app storage, app '.$appid.', user not logged in', OC_Log::ERROR);
 				return false;
 			}
 		}else{
