@@ -28,6 +28,21 @@ class OC_Helper {
 	private static $mimetypes=array();
 	private static $tmpFiles=array();
 
+	/**
+	 * @brief Creates an url using a defined route
+	 * @param $route
+	 * @param array $parameters
+	 * @return
+	 * @internal param array $args with param=>value, will be appended to the returned url
+	 * @returns the url
+	 *
+	 * Returns a url to the given app and file.
+	 */
+	public static function linkToRoute( $route, $parameters = array() ) {
+		$urlLinkTo = OC::getRouter()->generate($route, $parameters);
+		return $urlLinkTo;
+	}
+
 	/**
 	 * @brief Creates an url
 	 * @param string $app app
@@ -44,8 +59,8 @@ class OC_Helper {
 			// Check if the app is in the app folder
 			if( $app_path && file_exists( $app_path.'/'.$file )) {
 				if(substr($file, -3) == 'php' || substr($file, -3) == 'css') {
-					$urlLinkTo =  OC::$WEBROOT . '/?app=' . $app;
-					$urlLinkTo .= ($file!='index.php')?'&getfile=' . urlencode($file):'';
+					$urlLinkTo =  OC::$WEBROOT . '/index.php/apps/' . $app;
+					$urlLinkTo .= ($file!='index.php') ? '/' . $file : '';
 				}else{
 					$urlLinkTo =  OC_App::getAppWebPath($app) . '/' . $file;
 				}
@@ -63,11 +78,8 @@ class OC_Helper {
 			}
 		}
 
-		if (!empty($args)) {
-			$urlLinkTo .= '?';
-			foreach($args as $k => $v) {
-				$urlLinkTo .= '&'.$k.'='.urlencode($v);
-			}
+		if ($args && $query = http_build_query($args, '', '&')) {
+			$urlLinkTo .= '?'.$query;
 		}
 
 		return $urlLinkTo;
@@ -114,23 +126,27 @@ class OC_Helper {
 	/**
 	 * @brief Creates an absolute url for remote use
 	 * @param string $service id
+	 * @param bool $add_slash
 	 * @return string the url
 	 *
 	 * Returns a absolute url to the given service.
 	 */
 	public static function linkToRemote( $service, $add_slash = true ) {
-		return self::makeURLAbsolute(self::linkToRemoteBase($service)) . (($add_slash && $service[strlen($service)-1]!='/')?'/':'');
+		return self::makeURLAbsolute(self::linkToRemoteBase($service))
+			. (($add_slash && $service[strlen($service)-1]!='/')?'/':'');
 	}
 
 	/**
 	 * @brief Creates an absolute url for public use
 	 * @param string $service id
+	 * @param bool $add_slash
 	 * @return string the url
 	 *
 	 * Returns a absolute url to the given service.
 	 */
 	public static function linkToPublic($service, $add_slash = false) {
-		return self::linkToAbsolute( '', 'public.php') . '?service=' . $service . (($add_slash && $service[strlen($service)-1]!='/')?'/':'');
+		return self::linkToAbsolute( '', 'public.php') . '?service=' . $service
+			. (($add_slash && $service[strlen($service)-1]!='/')?'/':'');
 	}
 
 	/**
@@ -176,8 +192,9 @@ class OC_Helper {
 		if(isset($alias[$mimetype])) {
 			$mimetype=$alias[$mimetype];
 		}
-		// Replace slash with a minus
+		// Replace slash and backslash with a minus
 		$mimetype = str_replace( "/", "-", $mimetype );
+		$mimetype = str_replace( "\\", "-", $mimetype );
 
 		// Is it a dir?
 		if( $mimetype == "dir" ) {
@@ -189,7 +206,7 @@ class OC_Helper {
 			return OC::$WEBROOT."/core/img/filetypes/$mimetype.png";
 		}
 		//try only the first part of the filetype
-		$mimetype=substr($mimetype,0,strpos($mimetype,'-'));
+		$mimetype=substr($mimetype, 0, strpos($mimetype, '-'));
 		if( file_exists( OC::$SERVERROOT."/core/img/filetypes/$mimetype.png" )) {
 			return OC::$WEBROOT."/core/img/filetypes/$mimetype.png";
 		}
@@ -206,6 +223,10 @@ class OC_Helper {
 	 * Makes 2048 to 2 kB.
 	 */
 	public static function humanFileSize( $bytes ) {
+		if( $bytes < 0 ) {
+			$l = OC_L10N::get('lib');
+			return $l->t("couldn't be determined");
+		}
 		if( $bytes < 1024 ) {
 			return "$bytes B";
 		}
@@ -274,18 +295,18 @@ class OC_Helper {
 			if($file != '.' && $file != '..') {
 				$fullpath = $path.'/'.$file;
 				if(is_link($fullpath))
-					return FALSE;
+					return false;
 				elseif(!is_dir($fullpath) && !@chmod($fullpath, $filemode))
-						return FALSE;
+						return false;
 				elseif(!self::chmodr($fullpath, $filemode))
-					return FALSE;
+					return false;
 			}
 		}
 		closedir($dh);
 		if(@chmod($path, $filemode))
-			return TRUE;
+			return true;
 		else
-			return FALSE;
+			return false;
 	}
 
 	/**
@@ -305,7 +326,7 @@ class OC_Helper {
 					self::copyr("$src/$file", "$dest/$file");
 				}
 			}
-		}elseif(file_exists($src) && !OC_Filesystem::isFileBlacklisted($src)) {
+		}elseif(file_exists($src) && !\OC\Files\Filesystem::isFileBlacklisted($src)) {
 			copy($src, $dest);
 		}
 	}
@@ -341,29 +362,30 @@ class OC_Helper {
 	 * does NOT work for ownClouds filesystem, use OC_FileSystem::getMimeType instead
 	 */
 	static function getMimeType($path) {
-		$isWrapped=(strpos($path,'://')!==false) and (substr($path,0,7)=='file://');
+		$isWrapped=(strpos($path, '://')!==false) and (substr($path, 0, 7)=='file://');
 
 		if (@is_dir($path)) {
 			// directories are easy
 			return "httpd/unix-directory";
 		}
 
-		if(strpos($path,'.')) {
+		if(strpos($path, '.')) {
 			//try to guess the type by the file extension
-			if(!self::$mimetypes || self::$mimetypes != include('mimetypes.list.php')) {
-				self::$mimetypes=include('mimetypes.list.php');
+			if(!self::$mimetypes || self::$mimetypes != include 'mimetypes.list.php') {
+				self::$mimetypes=include 'mimetypes.list.php';
 			}
 			$extension=strtolower(strrchr(basename($path), "."));
-			$extension=substr($extension,1);//remove leading .
+			$extension=substr($extension, 1);//remove leading .
 			$mimeType=(isset(self::$mimetypes[$extension]))?self::$mimetypes[$extension]:'application/octet-stream';
 		}else{
 			$mimeType='application/octet-stream';
 		}
 
-		if($mimeType=='application/octet-stream' and function_exists('finfo_open') and function_exists('finfo_file') and $finfo=finfo_open(FILEINFO_MIME)) {
-			$info = @strtolower(finfo_file($finfo,$path));
+		if($mimeType=='application/octet-stream' and function_exists('finfo_open')
+			and function_exists('finfo_file') and $finfo=finfo_open(FILEINFO_MIME)) {
+			$info = @strtolower(finfo_file($finfo, $path));
 			if($info) {
-				$mimeType=substr($info,0,strpos($info,';'));
+				$mimeType=substr($info, 0, strpos($info, ';'));
 			}
 			finfo_close($finfo);
 		}
@@ -373,20 +395,14 @@ class OC_Helper {
 		}
 		if (!$isWrapped and $mimeType=='application/octet-stream' && OC_Helper::canExecute("file")) {
 			// it looks like we have a 'file' command,
-			// lets see it it does have mime support
+			// lets see if it does have mime support
 			$path=escapeshellarg($path);
-			$fp = popen("file -i -b $path 2>/dev/null", "r");
+			$fp = popen("file -b --mime-type $path 2>/dev/null", "r");
 			$reply = fgets($fp);
 			pclose($fp);
 
-			//trim the character set from the end of the response
-			$mimeType=substr($reply,0,strrpos($reply,' '));
-			$mimeType=substr($mimeType,0,strrpos($mimeType,"\n"));
-
-			//trim ;
-			if (strpos($mimeType, ';') !== false) {
-				$mimeType = strstr($mimeType, ';', true);
-			}
+			//trim the newline
+			$mimeType = trim($reply);
 
 		}
 		return $mimeType;
@@ -403,8 +419,8 @@ class OC_Helper {
 			return finfo_buffer($finfo, $data);
 		}else{
 			$tmpFile=OC_Helper::tmpFile();
-			$fh=fopen($tmpFile,'wb');
-			fwrite($fh,$data,8024);
+			$fh=fopen($tmpFile, 'wb');
+			fwrite($fh, $data, 8024);
 			fclose($fh);
 			$mime=self::getMimeType($tmpFile);
 			unset($tmpFile);
@@ -423,14 +439,16 @@ class OC_Helper {
 	//FIXME: should also check for value validation (i.e. the email is an email).
 	public static function init_var($s, $d="") {
 		$r = $d;
-		if(isset($_REQUEST[$s]) && !empty($_REQUEST[$s]))
-			$r = stripslashes(htmlspecialchars($_REQUEST[$s]));
+		if(isset($_REQUEST[$s]) && !empty($_REQUEST[$s])) {
+			$r = OC_Util::sanitizeHTML($_REQUEST[$s]);
+		}
 
 		return $r;
 	}
 
 	/**
-	 * returns "checked"-attribute if request contains selected radio element OR if radio element is the default one -- maybe?
+	 * returns "checked"-attribute if request contains selected radio element
+	 * OR if radio element is the default one -- maybe?
 	 * @param string $s Name of radio-button element name
 	 * @param string $v Value of current radio-button element
 	 * @param string $d Value of default radio-button element
@@ -441,12 +459,14 @@ class OC_Helper {
 	}
 
 	/**
-	* detect if a given program is found in the search PATH
-	*
-	* @param  string  $program name
-	* @param  string  $optional search path, defaults to $PATH
-	* @return bool    true if executable program found in path
-	*/
+	 * detect if a given program is found in the search PATH
+	 *
+	 * @param $name
+	 * @param bool $path
+	 * @internal param string $program name
+	 * @internal param string $optional search path, defaults to $PATH
+	 * @return bool    true if executable program found in path
+	 */
 	public static function canExecute($name, $path = false) {
 		// path defaults to PATH from environment if not set
 		if ($path === false) {
@@ -466,16 +486,16 @@ class OC_Helper {
 		$dirs = explode(PATH_SEPARATOR, $path);
 		// WARNING : We have to check if open_basedir is enabled :
 		$obd = ini_get('open_basedir');
-		if($obd != "none"){
+		if($obd != "none") {
 			$obd_values = explode(PATH_SEPARATOR, $obd);
-			if(count($obd_values) > 0 and $obd_values[0]){
+			if(count($obd_values) > 0 and $obd_values[0]) {
 				// open_basedir is in effect !
 				// We need to check if the program is in one of these dirs :
 				$dirs = $obd_values;
 			}
 		}
-		foreach($dirs as $dir){
-			foreach($exts as $ext){
+		foreach($dirs as $dir) {
+			foreach($exts as $ext) {
 				if($check_fn("$dir/$name".$ext))
 					return true;
 			}
@@ -489,15 +509,20 @@ class OC_Helper {
 	 * @param resource $target
 	 * @return int the number of bytes copied
 	 */
-	public static function streamCopy($source,$target) {
+	public static function streamCopy($source, $target) {
 		if(!$source or !$target) {
 			return false;
 		}
-		$count=0;
+		$result = true;
+		$count = 0;
 		while(!feof($source)) {
-			$count+=fwrite($target,fread($source,8192));
+			if ( ( $c = fwrite($target, fread($source, 8192)) ) === false) {
+				$result = false;
+			} else {
+				$count += $c;
+			}
 		}
-		return $count;
+		return array($count, $result);
 	}
 
 	/**
@@ -509,12 +534,33 @@ class OC_Helper {
 	 */
 	public static function tmpFile($postfix='') {
 		$file=get_temp_dir().'/'.md5(time().rand()).$postfix;
-		$fh=fopen($file,'w');
+		$fh=fopen($file, 'w');
 		fclose($fh);
 		self::$tmpFiles[]=$file;
 		return $file;
 	}
 
+	/**
+	 * create a temporary file with an unique filename. It will not be deleted
+	 * automatically
+	 * @param string $postfix
+	 * @return string
+	 *
+	 */
+	public static function tmpFileNoClean($postfix='') {
+		$tmpDirNoClean=get_temp_dir().'/oc-noclean/';
+		if (!file_exists($tmpDirNoClean) || !is_dir($tmpDirNoClean)) {
+			if (file_exists($tmpDirNoClean)) {
+				unlink($tmpDirNoClean);
+			}
+			mkdir($tmpDirNoClean);
+		}
+		$file=$tmpDirNoClean.md5(time().rand()).$postfix;
+		$fh=fopen($file, 'w');
+		fclose($fh);
+		return $file;
+	}
+
 	/**
 	 * create a temporary folder with an unique filename
 	 * @return string
@@ -550,6 +596,16 @@ class OC_Helper {
 		}
 	}
 
+	/**
+	 * remove all files created by self::tmpFileNoClean
+	 */
+	public static function cleanTmpNoClean() {
+		$tmpDirNoCleanFile=get_temp_dir().'/oc-noclean/';
+		if(file_exists($tmpDirNoCleanFile)) {
+			self::rmdirr($tmpDirNoCleanFile);
+		}
+	}
+
 	/**
 	* Adds a suffix to the name in case the file exists
 	*
@@ -571,7 +627,7 @@ class OC_Helper {
 
 		$newpath = $path . '/' . $filename;
 		$counter = 2;
-		while (OC_Filesystem::file_exists($newpath)) {
+		while (\OC\Files\Filesystem::file_exists($newpath)) {
 			$newname = $name . ' (' . $counter . ')' . $ext;
 			$newpath = $path . '/' . $newname;
 			$counter++;
@@ -580,37 +636,17 @@ class OC_Helper {
 		return $newpath;
 	}
 
-	/*
-	 * checks if $sub is a subdirectory of $parent
+	/**
+	 * @brief Checks if $sub is a subdirectory of $parent
 	 *
 	 * @param string $sub
 	 * @param string $parent
 	 * @return bool
 	 */
 	public static function issubdirectory($sub, $parent) {
-		if($sub == null || $sub == '' || $parent == null || $parent == '') {
-			return false;
+		if (strpos(realpath($sub), realpath($parent)) === 0) {
+			return true;
 		}
-		$realpath_sub = realpath($sub);
-		$realpath_parent = realpath($parent);
-		if(($realpath_sub == false && substr_count($realpath_sub, './') != 0) || ($realpath_parent == false && substr_count($realpath_parent, './') != 0)) { //it checks for  both ./ and ../
-			return false;
-		}
-		if($realpath_sub && $realpath_sub != '' && $realpath_parent && $realpath_parent != '') {
-			if(substr($realpath_sub, 0, strlen($realpath_parent)) == $realpath_parent) {
-				return true;
-			}
-		}else{
-			if(substr($sub, 0, strlen($parent)) == $parent) {
-				return true;
-			}
-		}
-		/*echo 'SUB: ' . $sub . "\n";
-		echo 'PAR: ' . $parent . "\n";
-		echo 'REALSUB: ' . $realpath_sub . "\n";
-		echo 'REALPAR: ' . $realpath_parent . "\n";
-		echo substr($realpath_sub, 0, strlen($realpath_parent));
-		exit;*/
 		return false;
 	}
 
@@ -636,22 +672,22 @@ class OC_Helper {
 	}
 
 	/**
-	* @brief replaces a copy of string delimited by the start and (optionally) length parameters with the string given in replacement.
-	*
-	* @param string $input The input string. .Opposite to the PHP build-in function does not accept an array.
-	* @param string $replacement The replacement string.
-	* @param int $start If start is positive, the replacing will begin at the start'th offset into string. If start is negative, the replacing will begin at the start'th character from the end of string.
-	* @param int $length Length of the part to be replaced
-	* @param string $encoding The encoding parameter is the character encoding. Defaults to UTF-8
-	* @return string
-	*
-	*/
+	 * @brief replaces a copy of string delimited by the start and (optionally) length parameters with the string given in replacement.
+	 *
+	 * @param $string
+	 * @param string $replacement The replacement string.
+	 * @param int $start If start is positive, the replacing will begin at the start'th offset into string. If start is negative, the replacing will begin at the start'th character from the end of string.
+	 * @param int $length Length of the part to be replaced
+	 * @param string $encoding The encoding parameter is the character encoding. Defaults to UTF-8
+	 * @internal param string $input The input string. .Opposite to the PHP build-in function does not accept an array.
+	 * @return string
+	 */
 	public static function mb_substr_replace($string, $replacement, $start, $length = null, $encoding = 'UTF-8') {
 		$start = intval($start);
 		$length = intval($length);
 		$string = mb_substr($string, 0, $start, $encoding) .
-		          $replacement .
-		          mb_substr($string, $start+$length, mb_strlen($string, 'UTF-8')-$start, $encoding);
+			$replacement .
+			mb_substr($string, $start+$length, mb_strlen($string, 'UTF-8')-$start, $encoding);
 
 		return $string;
 	}
@@ -672,7 +708,7 @@ class OC_Helper {
 		$length = mb_strlen($search, $encoding);
 		while(($i = mb_strrpos($subject, $search, $offset, $encoding)) !== false ) {
 			$subject = OC_Helper::mb_substr_replace($subject, $replace, $i, $length);
-			$offset = $i - mb_strlen($subject, $encoding) - 1;
+			$offset = $i - mb_strlen($subject, $encoding);
 			$count++;
 		}
 		return $subject;
@@ -703,4 +739,79 @@ class OC_Helper {
 
 		return false;
 	}
+
+	/**
+	 * Shortens str to maxlen by replacing characters in the middle with '...', eg.
+	 * ellipsis('a very long string with lots of useless info to make a better example', 14) becomes 'a very ...example'
+	 * @param string $str the string
+	 * @param string $maxlen the maximum length of the result
+	 * @return string with at most maxlen characters
+	 */
+	public static function ellipsis($str, $maxlen) {
+		if (strlen($str) > $maxlen) {
+			$characters = floor($maxlen / 2);
+			return substr($str, 0, $characters) . '...' . substr($str, -1 * $characters);
+		}
+		return $str;
+	}
+
+	/**
+	 * @brief calculates the maximum upload size respecting system settings, free space and user quota
+	 *
+	 * @param $dir the current folder where the user currently operates
+	 * @return number of bytes representing
+	 */
+	public static function maxUploadFilesize($dir) {
+		$upload_max_filesize = OCP\Util::computerFileSize(ini_get('upload_max_filesize'));
+		$post_max_size = OCP\Util::computerFileSize(ini_get('post_max_size'));
+		$maxUploadFilesize = min($upload_max_filesize, $post_max_size);
+
+		$freeSpace = \OC\Files\Filesystem::free_space($dir);
+		if($freeSpace !== \OC\Files\FREE_SPACE_UNKNOWN){
+			$freeSpace = max($freeSpace, 0);
+
+			return min($maxUploadFilesize, $freeSpace);
+		} else {
+			return $maxUploadFilesize;
+		}
+	}
+
+	/**
+	 * Checks if a function is available
+	 * @param string $function_name
+	 * @return bool
+	 */
+	public static function is_function_enabled($function_name) {
+		if (!function_exists($function_name)) {
+			return false;
+		}
+		$disabled = explode(', ', ini_get('disable_functions'));
+		if (in_array($function_name, $disabled)) {
+			return false;
+		}
+		$disabled = explode(', ', ini_get('suhosin.executor.func.blacklist'));
+		if (in_array($function_name, $disabled)) {
+			return false;
+		}
+		return true;
+	}
+
+	/**
+	 * Calculate the disc space
+	 */
+	public static function getStorageInfo() {
+		$rootInfo = \OC\Files\Filesystem::getFileInfo('/');
+		$used = $rootInfo['size'];
+		if ($used < 0) {
+			$used = 0;
+		}
+		$free = \OC\Files\Filesystem::free_space();
+		$total = $free + $used;
+		if ($total == 0) {
+			$total = 1; // prevent division by zero
+		}
+		$relative = round(($used / $total) * 10000) / 100;
+
+		return array('free' => $free, 'used' => $used, 'total' => $total, 'relative' => $relative);
+	}
 }
